from typing import List, Optional, Dict, Any
from sqlalchemy.orm import Session

from app.models.task import Task
from app.schemas.task import TaskCreate, TaskUpdate

def get_task(db: Session, task_id: int) -> Optional[Task]:
    return db.query(Task).filter(Task.id == task_id).first()

def get_tasks(
    db: Session, 
    skip: int = 0, 
    limit: int = 100,
    filters: Optional[Dict[str, Any]] = None
) -> List[Task]:
    query = db.query(Task)
    
    if filters:
        if status := filters.get("status"):
            query = query.filter(Task.status == status)
        if priority := filters.get("priority"):
            query = query.filter(Task.priority == priority)
        if completed := filters.get("completed"):
            query = query.filter(Task.completed == completed)
    
    return query.order_by(Task.created_at.desc()).offset(skip).limit(limit).all()

def get_tasks_count(
    db: Session,
    filters: Optional[Dict[str, Any]] = None
) -> int:
    query = db.query(Task)
    
    if filters:
        if status := filters.get("status"):
            query = query.filter(Task.status == status)
        if priority := filters.get("priority"):
            query = query.filter(Task.priority == priority)
        if completed := filters.get("completed"):
            query = query.filter(Task.completed == completed)
    
    return query.count()

def create_task(db: Session, task: TaskCreate) -> Task:
    db_task = Task(
        title=task.title,
        description=task.description,
        status=task.status,
        priority=task.priority,
        due_date=task.due_date,
        completed=task.completed
    )
    db.add(db_task)
    db.commit()
    db.refresh(db_task)
    return db_task

def update_task(db: Session, task_id: int, task: TaskUpdate) -> Optional[Task]:
    db_task = get_task(db, task_id)
    if not db_task:
        return None
    
    update_data = task.dict(exclude_unset=True)
    for key, value in update_data.items():
        setattr(db_task, key, value)
    
    db.commit()
    db.refresh(db_task)
    return db_task

def delete_task(db: Session, task_id: int) -> bool:
    db_task = get_task(db, task_id)
    if not db_task:
        return False
    
    db.delete(db_task)
    db.commit()
    return True